class ModelServerError(Exception):
    def __init__(self, msg):
        super().__init__(msg)


class ModelNotFound(ModelServerError):
    def __init__(self, name: str, version: str = None):
        msg = f"Model {name} not found"
        if version is not None:
            msg = f"Model {name} with version {version} not found"

        super().__init__(msg)



import asyncio

from ..errors import ModelNotFound
from ..model import NuxModel
from ..repository import Repository
from ..settings import ModelConfig
from ..types import (
    RepositoryIndexRequest,
    RepositoryIndexResponse,
    RepositoryIndexResponseItem,
    State,
)


class RepositoryHandler:
    def __init__(self, repository: Repository):
        self._repository = repository

    async def index(self, payload: RepositoryIndexRequest) -> RepositoryIndexResponse:
        model_configs = await self._repository.list()
        items = await asyncio.gather(*(self._to_item(config) for config in model_configs))

        def returnable(item):
            if payload.ready is None:
                return True

            if not payload.ready:
                return item.state != State.READY

            return item.state == State.READY

        return RepositoryIndexResponse(__root__=list(filter(returnable, items)))

    async def _to_item(self, config: ModelConfig) -> RepositoryIndexResponseItem:
        item = RepositoryIndexResponseItem(
            name=config.name,
            state=State.UNKNOWN,
            reason="",
        )

        item.state = await self._get_state(config)
        item.version = config.version
        return item

    async def _get_state(self, config: ModelConfig) -> State:
        try:
            model = await self._repository.get_model(config.name, config.version)
            if model.ready:
                return State.READY
        except ModelNotFound:
            return State.UNAVAILABLE

        # TODO(yan): Support LOADING/UNLOADING states
        return State.UNKNOWN

    async def load(self, name: str) -> bool:
        model_config = await self._repository.find(name)

        # TODO(yan): Support other model implementation
        return await self._repository.load(NuxModel(model_config))

    async def unload(self, name: str) -> bool:
        return await self._repository.unload(name)



import asyncio
from typing import Callable, Dict, List, Optional

from .errors import ModelNotFound
from .model import Model
from .registry import Registry
from .settings import ModelConfig


class Repository:
    def __init__(
        self,
        registries: List[Registry],
        on_load: Optional[Callable[[Model], None]] = None,
        on_unload: Optional[Callable[[Model], None]] = None,
    ):
        self._registries = registries
        # Ready models which are loaded. Schema: {model_name: {model_version: Model}}
        self._models: Dict[str, Dict[str, Model]] = {}

        self.on_load = on_load
        self.on_unload = on_unload

    @property
    def registries(self) -> List[Registry]:
        return self._registries

    async def load(self, model: Model) -> bool:
        """
        Load a specified model

        Register a model in the internal directory to maintain loaded models.
        """
        versions = self._models.setdefault(model.name, {})

        version = model.version or "default"

        if version in versions:
            # Do not load again when model is already ready
            return True

        # We load the specified model again even if it was already loaded
        versions[version] = model

        loaded = await model.load()
        if loaded and self.on_load:
            self.on_load(model)
        return loaded

    async def unload(self, name: str) -> bool:
        """
        Unload models with a specified name

        Unregister models in the internal directory to maintain loaded models.
        """
        if name not in self._models:
            raise ModelNotFound(name)

        versions = self._models.pop(name)
        for model in versions.values():
            await model.unload()
            if self.on_unload:
                self.on_unload(model)
        return True

    async def get_model(self, name: str, version: str = None) -> Model:
        """
        Get a specified loaded model with name and version
        """
        version = version or "default"

        if name not in self._models:
            raise ModelNotFound(name, version)

        if version not in self._models[name]:
            raise ModelNotFound(name, version)

        return self._models[name][version]

    async def get_models(self) -> List[Model]:
        """
        Get the specified loaded models
        """
        return [model for versions in self._models.values() for model in versions.values()]

    async def list(self) -> List[ModelConfig]:
        """
        Get the (loaded + unloaded) model configs from several registries
        """
        return sum(
            list(await asyncio.gather(*(registry.list() for registry in self._registries))),
            [],
        )

    async def find(self, name: str) -> ModelConfig:
        """
        Find a (loaded + unloaded) model configs from several registries
        """
        # TODO(yan): Choose specific version instead of first item
        for registry in self._registries:
            try:
                config = await registry.find(name)
                return config
            except ModelNotFound:
                pass

        raise ModelNotFound(name)



import uvicorn

from ...handlers import PredictHandler, RepositoryHandler
from ...settings import RESTServerConfig
from .app import create


class RESTServer:
    def __init__(
        self,
        config: RESTServerConfig,
        predict_handler: PredictHandler,
        repository_handler: RepositoryHandler,
    ):
        self._config = config
        self._app = create(
            self._config,
            predict_handler=predict_handler,
            repository_handler=repository_handler,
        )

    async def start(self):
        class NoSignalServer(uvicorn.Server):
            def install_signal_handlers(self):
                # Delegate signal handling to ModelServer
                pass

        self._server = NoSignalServer(
            uvicorn.Config(
                self._app,
                host=self._config.host,
                port=self._config.port,
                workers=self._config.workers,
            )
        )
        await self._server.serve()

    async def stop(self):
        self._server.handle_exit(sig=None, frame=None)



# generated by datamodel-codegen:
#   filename:  predict.yaml
#   timestamp: 2021-07-26T09:49:58+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel, Extra, Field


class MetadataServerResponse(BaseModel):
    name: str
    version: str
    extensions: List[str]


class MetadataServerErrorResponse(BaseModel):
    error: str


class Tags(BaseModel):
    class Config:
        extra = Extra.allow

    content_type: Optional[str] = None


class MetadataTensor(BaseModel):
    name: str
    datatype: str
    shape: List[int]
    tags: Optional[Tags] = None


class MetadataModelErrorResponse(BaseModel):
    error: str


class Parameters(BaseModel):
    class Config:
        extra = Extra.allow

    content_type: Optional[str] = None


class TensorData(BaseModel):
    __root__: Any = Field(..., title='tensor_data')

    def __iter__(self):
        return iter(self.__root__)

    def __getitem__(self, idx):
        return self.__root__[idx]

    def __len__(self):
        return len(self.__root__)


class RequestOutput(BaseModel):
    name: str
    parameters: Optional[Parameters] = None


class ResponseOutput(BaseModel):
    name: str
    shape: List[int]
    datatype: str
    parameters: Optional[Parameters] = None
    data: TensorData


class InferenceResponse(BaseModel):
    model_name: str
    model_version: Optional[str] = None
    id: Optional[str] = None
    parameters: Optional[Parameters] = None
    outputs: List[ResponseOutput]


class InferenceErrorResponse(BaseModel):
    error: Optional[str] = None


class MetadataModelResponse(BaseModel):
    name: str
    versions: Optional[List[str]] = None
    platform: str
    inputs: Optional[List[MetadataTensor]] = None
    outputs: Optional[List[MetadataTensor]] = None


class RequestInput(BaseModel):
    name: str
    shape: List[int]
    datatype: str
    parameters: Optional[Parameters] = None
    data: TensorData


class InferenceRequest(BaseModel):
    id: Optional[str] = None
    parameters: Optional[Parameters] = None
    inputs: List[RequestInput]
    outputs: Optional[List[RequestOutput]] = None



"""FuriosaAI model server interacting Furiosa NPU"""

from .errors import ModelNotFound, ModelServerError
from .model import Model, NuxModel
from .server import ModelServer
from .settings import ModelConfig, ServerConfig

__all__ = [
    "Model",
    "ModelConfig",
    "ModelNotFound",
    "ModelServer",
    "ModelServerError",
    "NuxModel",
    "ServerConfig",
]



"""
Command line interface for FuriosaAI model server
"""

import enum
import logging
from typing import Any, Dict

from pydantic import ValidationError
import typer

from furiosa.common.thread import synchronous

from ...server import ModelServer
from ...settings import ModelConfig, RESTServerConfig, ServerConfig
from ...utils.loader import load_model_config, load_server_config


class LogLevel(str, enum.Enum):
    """
    Log level enum for typer choice parameter
    """

    ERROR = "ERROR"
    INFO = "INFO"
    WARN = "WARN"
    DEBUG = "DEBUG"
    TRACE = "TRACE"


def _display_error_loc(error: Dict[str, Any]) -> str:
    return ' -> '.join(str(e) for e in error['loc'])


@synchronous
async def start(
    log_level: LogLevel = LogLevel.INFO,
    model_name: str = typer.Option(None, help="Model name"),
    model_path: str = typer.Option(None, help="Path to a model file (tflite, onnx are supported)"),
    model_version: str = typer.Option("default", help="Model version"),
    host: str = typer.Option("0.0.0.0", help="IPv4 address to bind"),
    http_port: int = typer.Option(8080, help="HTTP port to bind"),
    model_config: typer.FileText = typer.Option(None, help="Path to a model config file"),
    server_config: typer.FileText = typer.Option(None, help="Path to a server config file"),
):
    """
    Start serving models from FuriosaAI model server
    """
    logging.basicConfig(level=log_level.value)

    if not model_config:
        if not model_path:
            typer.echo("Missing option '--model-path'")
            raise typer.Exit(1)

        if not model_name:
            typer.echo("ERROR: missing option '--model-name'")
            raise typer.Exit(1)
    else:
        if model_path:
            typer.echo("ERROR: '--model-path' cannot be used with '--model-config'")
            raise typer.Exit(1)

        if model_name:
            typer.echo("ERROR: '--model-name' cannot be used with '--model-config'")
            raise typer.Exit(1)

    if server_config:
        config = load_server_config(server_config)
    else:
        rest_config = RESTServerConfig(
            host=host, port=http_port, debug=(log_level == LogLevel.DEBUG)
        )
        config = ServerConfig(rest_server_config=rest_config)

    if model_config:
        try:
            model_configs = load_model_config(model_config)
        except ValidationError as e:
            first_err = e.errors()[0]
            typer.echo(f"ERROR: {_display_error_loc(first_err)} {first_err['msg']}")
            raise typer.Exit(1)
    else:
        model_configs = [ModelConfig(model=model_path, name=model_name, version=model_version)]

    await ModelServer(config, model_configs).start()


def main():
    """
    Entry point for command line interface
    """
    typer.run(start)


if __name__ == "__main__":
    main()



from ..repository import Repository
from ..settings import ServerConfig
from ..types import (
    InferenceRequest,
    InferenceResponse,
    MetadataModelResponse,
    MetadataServerResponse,
)


class PredictHandler:
    def __init__(self, config: ServerConfig, repository: Repository):
        self._config = config
        self._repository = repository

    async def live(self) -> bool:
        return True

    async def ready(self) -> bool:
        models = await self._repository.get_models()
        return all(model.ready for model in models)

    async def model_ready(self, name: str, version: str = None) -> bool:
        model = await self._repository.get_model(name, version)
        return model.ready

    async def metadata(self) -> MetadataServerResponse:
        return MetadataServerResponse(
            name=self._config.server_name,
            version=self._config.server_version,
            extensions=self._config.extensions,
        )

    async def model_metadata(self, name: str, version: str = None) -> MetadataModelResponse:
        model = await self._repository.get_model(name, version)
        return await model.metadata()

    async def infer(
        self, payload: InferenceRequest, name: str, version: str = None
    ) -> InferenceResponse:
        model = await self._repository.get_model(name, version)

        prediction = await model.predict(payload)
        prediction.id = payload.id

        return prediction



from fastapi import status
from fastapi.responses import Response

from ...handlers import PredictHandler, RepositoryHandler
from ...types import (
    InferenceRequest,
    InferenceResponse,
    MetadataModelResponse,
    MetadataServerResponse,
    RepositoryIndexRequest,
    RepositoryIndexResponse,
)


class BooleanResponse(Response):
    def __init__(self, response: bool, error_code: int = status.HTTP_400_BAD_REQUEST):
        status_code = status.HTTP_200_OK if response else error_code
        super().__init__(status_code=status_code)


class ModelEndpoints:
    def __init__(self, handler: PredictHandler):
        self._handler = handler

    async def live(self) -> Response:
        return BooleanResponse(await self._handler.live())

    async def ready(self) -> Response:
        return BooleanResponse(await self._handler.ready())

    async def model_ready(self, model_name: str, model_version: str = None) -> Response:
        return BooleanResponse(await self._handler.model_ready(model_name, model_version))

    async def metadata(self) -> MetadataServerResponse:
        return await self._handler.metadata()

    async def model_metadata(
        self, model_name: str, model_version: str = None
    ) -> MetadataModelResponse:
        return await self._handler.model_metadata(model_name, model_version)

    async def infer(
        self,
        payload: InferenceRequest,
        model_name: str,
        model_version: str = None,
    ) -> InferenceResponse:
        return await self._handler.infer(payload, model_name, model_version)


class ModelRepositoryEndpoints:
    def __init__(self, handler: RepositoryHandler):
        self._handler = handler

    async def index(self, payload: RepositoryIndexRequest) -> RepositoryIndexResponse:
        return await self._handler.index(payload)

    async def load(self, model_name: str) -> Response:
        return BooleanResponse(await self._handler.load(name=model_name))

    async def unload(self, model_name: str) -> Response:
        return BooleanResponse(await self._handler.unload(name=model_name))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

# FIXME(yan): This is manually modified
from . import predict_pb2 as predict__pb2


class GRPCInferenceServiceStub(object):
    """Inference Server GRPC endpoints."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ServerLive = channel.unary_unary(
            '/inference.GRPCInferenceService/ServerLive',
            request_serializer=predict__pb2.ServerLiveRequest.SerializeToString,
            response_deserializer=predict__pb2.ServerLiveResponse.FromString,
        )
        self.ServerReady = channel.unary_unary(
            '/inference.GRPCInferenceService/ServerReady',
            request_serializer=predict__pb2.ServerReadyRequest.SerializeToString,
            response_deserializer=predict__pb2.ServerReadyResponse.FromString,
        )
        self.ModelReady = channel.unary_unary(
            '/inference.GRPCInferenceService/ModelReady',
            request_serializer=predict__pb2.ModelReadyRequest.SerializeToString,
            response_deserializer=predict__pb2.ModelReadyResponse.FromString,
        )
        self.ServerMetadata = channel.unary_unary(
            '/inference.GRPCInferenceService/ServerMetadata',
            request_serializer=predict__pb2.ServerMetadataRequest.SerializeToString,
            response_deserializer=predict__pb2.ServerMetadataResponse.FromString,
        )
        self.ModelMetadata = channel.unary_unary(
            '/inference.GRPCInferenceService/ModelMetadata',
            request_serializer=predict__pb2.ModelMetadataRequest.SerializeToString,
            response_deserializer=predict__pb2.ModelMetadataResponse.FromString,
        )
        self.ModelInfer = channel.unary_unary(
            '/inference.GRPCInferenceService/ModelInfer',
            request_serializer=predict__pb2.ModelInferRequest.SerializeToString,
            response_deserializer=predict__pb2.ModelInferResponse.FromString,
        )


class GRPCInferenceServiceServicer(object):
    """Inference Server GRPC endpoints."""

    def ServerLive(self, request, context):
        """The ServerLive API indicates if the inference server is able to receive
        and respond to metadata and inference requests.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ServerReady(self, request, context):
        """The ServerReady API indicates if the server is ready for inferencing."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModelReady(self, request, context):
        """The ModelReady API indicates if a specific model is ready for inferencing."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ServerMetadata(self, request, context):
        """The ServerMetadata API provides information about the server. Errors are
        indicated by the google.rpc.Status returned for the request. The OK code
        indicates success and other codes indicate failure.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModelMetadata(self, request, context):
        """The per-model metadata API provides information about a model. Errors are
        indicated by the google.rpc.Status returned for the request. The OK code
        indicates success and other codes indicate failure.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModelInfer(self, request, context):
        """The ModelInfer API performs inference using the specified model. Errors are
        indicated by the google.rpc.Status returned for the request. The OK code
        indicates success and other codes indicate failure.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GRPCInferenceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'ServerLive': grpc.unary_unary_rpc_method_handler(
            servicer.ServerLive,
            request_deserializer=predict__pb2.ServerLiveRequest.FromString,
            response_serializer=predict__pb2.ServerLiveResponse.SerializeToString,
        ),
        'ServerReady': grpc.unary_unary_rpc_method_handler(
            servicer.ServerReady,
            request_deserializer=predict__pb2.ServerReadyRequest.FromString,
            response_serializer=predict__pb2.ServerReadyResponse.SerializeToString,
        ),
        'ModelReady': grpc.unary_unary_rpc_method_handler(
            servicer.ModelReady,
            request_deserializer=predict__pb2.ModelReadyRequest.FromString,
            response_serializer=predict__pb2.ModelReadyResponse.SerializeToString,
        ),
        'ServerMetadata': grpc.unary_unary_rpc_method_handler(
            servicer.ServerMetadata,
            request_deserializer=predict__pb2.ServerMetadataRequest.FromString,
            response_serializer=predict__pb2.ServerMetadataResponse.SerializeToString,
        ),
        'ModelMetadata': grpc.unary_unary_rpc_method_handler(
            servicer.ModelMetadata,
            request_deserializer=predict__pb2.ModelMetadataRequest.FromString,
            response_serializer=predict__pb2.ModelMetadataResponse.SerializeToString,
        ),
        'ModelInfer': grpc.unary_unary_rpc_method_handler(
            servicer.ModelInfer,
            request_deserializer=predict__pb2.ModelInferRequest.FromString,
            response_serializer=predict__pb2.ModelInferResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'inference.GRPCInferenceService', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class GRPCInferenceService(object):
    """Inference Server GRPC endpoints."""

    @staticmethod
    def ServerLive(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ServerLive',
            predict__pb2.ServerLiveRequest.SerializeToString,
            predict__pb2.ServerLiveResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ServerReady(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ServerReady',
            predict__pb2.ServerReadyRequest.SerializeToString,
            predict__pb2.ServerReadyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ModelReady(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ModelReady',
            predict__pb2.ModelReadyRequest.SerializeToString,
            predict__pb2.ModelReadyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ServerMetadata(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ServerMetadata',
            predict__pb2.ServerMetadataRequest.SerializeToString,
            predict__pb2.ServerMetadataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ModelMetadata(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ModelMetadata',
            predict__pb2.ModelMetadataRequest.SerializeToString,
            predict__pb2.ModelMetadataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ModelInfer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.GRPCInferenceService/ModelInfer',
            predict__pb2.ModelInferRequest.SerializeToString,
            predict__pb2.ModelInferResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from collections.abc import MutableMapping
from io import TextIOWrapper
from typing import Dict, List, Union

import yaml

from furiosa.server.settings import ModelConfig, ServerConfig


def load_model_config(stream: Union[TextIOWrapper, Dict]) -> List[ModelConfig]:
    """
    Load model configs from opened file (file-like object) or Python dictionary
    """

    source = stream if isinstance(stream, MutableMapping) else yaml.safe_load(stream)
    return [ModelConfig.parse_obj(model) for model in source["model_config_list"]]


def load_server_config(stream: Union[TextIOWrapper, Dict]) -> ServerConfig:
    """
    Load a server config from opened file (file-like object) or Python dictionary
    """

    source = stream if isinstance(stream, MutableMapping) else yaml.safe_load(stream)
    return ServerConfig.parse_obj(source)



from .model_repository import (
    RepositoryIndexErrorResponse,
    RepositoryIndexRequest,
    RepositoryIndexResponse,
    RepositoryIndexResponseItem,
    RepositoryLoadErrorResponse,
    RepositoryUnloadErrorResponse,
    State,
)
from .predict import (
    InferenceErrorResponse,
    InferenceRequest,
    InferenceResponse,
    MetadataModelErrorResponse,
    MetadataModelResponse,
    MetadataServerErrorResponse,
    MetadataServerResponse,
    MetadataTensor,
    Parameters,
    RequestInput,
    RequestOutput,
    ResponseOutput,
    Tags,
    TensorData,
)

__all__ = [
    # Predict
    "MetadataServerResponse",
    "MetadataServerErrorResponse",
    "MetadataTensor",
    "MetadataModelErrorResponse",
    "Parameters",
    "Tags",
    "TensorData",
    "RequestOutput",
    "ResponseOutput",
    "InferenceResponse",
    "InferenceErrorResponse",
    "MetadataModelResponse",
    "RequestInput",
    "InferenceRequest",
    # Model Repository
    "RepositoryIndexRequest",
    "RepositoryIndexResponseItem",
    "State",
    "RepositoryIndexResponse",
    "RepositoryIndexErrorResponse",
    "RepositoryLoadErrorResponse",
    "RepositoryUnloadErrorResponse",
]



from fastapi import FastAPI, Request, status
from fastapi.responses import JSONResponse
from fastapi.routing import APIRoute

from ...errors import ModelServerError
from ...handlers import PredictHandler, RepositoryHandler
from ...settings import RESTServerConfig
from .endpoints import ModelEndpoints, ModelRepositoryEndpoints


def create(
    config: RESTServerConfig,
    predict_handler: PredictHandler,
    repository_handler: RepositoryHandler,
) -> FastAPI:
    endpoints = ModelEndpoints(predict_handler)
    model_repository_endpoints = ModelRepositoryEndpoints(repository_handler)

    routes = [
        # Model ready
        APIRoute(
            "/v2/models/{model_name}/ready",
            endpoints.model_ready,
        ),
        APIRoute(
            "/v2/models/{model_name}/versions/{model_version}/ready",
            endpoints.model_ready,
        ),
        # Model infer
        APIRoute(
            "/v2/models/{model_name}/infer",
            endpoints.infer,
            methods=["POST"],
        ),
        APIRoute(
            "/v2/models/{model_name}/versions/{model_version}/infer",
            endpoints.infer,
            methods=["POST"],
        ),
        # Model metadata
        APIRoute(
            "/v2/models/{model_name}",
            endpoints.model_metadata,
        ),
        APIRoute(
            "/v2/models/{model_name}/versions/{model_version}",
            endpoints.model_metadata,
        ),
        # Liveness and readiness
        APIRoute("/v2/health/live", endpoints.live),
        APIRoute("/v2/health/ready", endpoints.ready),
        # Server metadata
        APIRoute(
            "/v2",
            endpoints.metadata,
        ),
    ]

    routes += [
        # Model Repository API
        APIRoute(
            "/v2/repository/index",
            model_repository_endpoints.index,
            methods=["POST"],
        ),
        APIRoute(
            "/v2/repository/models/{model_name}/load",
            model_repository_endpoints.load,
            methods=["POST"],
        ),
        APIRoute(
            "/v2/repository/models/{model_name}/unload",
            model_repository_endpoints.unload,
            methods=["POST"],
        ),
    ]

    app = FastAPI(
        debug=config.debug,
        routes=routes,  # type: ignore
        exception_handlers={ModelServerError: exception_handler},  # type: ignore
    )

    return app


def exception_handler(request: Request, exc: ModelServerError) -> JSONResponse:
    return JSONResponse(status_code=status.HTTP_400_BAD_REQUEST, content=str(exc))



from abc import ABC, abstractmethod
import glob
import os
from typing import List

import yaml

from .errors import ModelNotFound
from .settings import FileRegistryConfig, ModelConfig


class Registry(ABC):
    @abstractmethod
    async def list(self) -> List[ModelConfig]:
        """
        Get the (loaded + unloaded) model configs
        """
        pass

    async def find(self, name: str) -> ModelConfig:
        """
        Find a (loaded + unloaded) model configs
        """
        # TODO(yan): Choose specific version instead of first item
        first = next(
            (config for config in await self.list() if config.name == name),
            None,
        )

        if first is None:
            raise ModelNotFound(name)

        return first


class InMemoryRegistry(Registry):
    def __init__(self, model_configs: List[ModelConfig] = []):
        self._model_configs = model_configs

    async def list(self) -> List[ModelConfig]:
        """
        Get the model configs in-memory
        """
        return self._model_configs

    def register(self, model_config: ModelConfig) -> bool:
        """
        Register a model config in this in-memory registry

        Returns:
            bool: True for success, False if the config is already existed
        """
        if model_config in self._model_configs:
            return False

        self._model_configs.append(model_config)
        return True

    def unregister(self, model_config: ModelConfig) -> bool:
        """
        Unregister a model config from this in-memory registry

        Returns:
            bool: True for success, False if the config is not existed
        """
        if model_config not in self._model_configs:
            return False

        self._model_configs.remove(model_config)
        return True


class FileRegistry(Registry):
    def __init__(self, config: FileRegistryConfig):
        self._config = config

    async def list(self) -> List[ModelConfig]:
        """
        Get the model configs from file with yaml suffix in the specified directory
        """
        pattern = os.path.join(self._config.repository_root, "*.yaml")
        matches = glob.glob(pattern, recursive=True)

        def load(config_file):
            models = yaml.safe_load(config_file)["model_config_list"]
            return [ModelConfig.parse_obj(model) for model in models]

        return sum((load(config) for config in matches), [])



"""
Configuration settings via Pydantic
"""

from typing import Dict, List, Optional, Union

from pydantic import BaseSettings, StrictBytes, StrictStr

from furiosa.common.utils import get_sdk_version

from .types import MetadataTensor

__version__ = get_sdk_version("furiosa.server")


class GRPCServerConfig(BaseSettings):
    """
    GRPC server configuration
    """

    host: str = "0.0.0.0"
    port: int = 8081
    workers: int = 1


class RESTServerConfig(BaseSettings):
    """
    Rest server configuration
    """

    host: str = "0.0.0.0"
    port: int = 8080
    workers: int = 1
    debug: bool = False


class FileRegistryConfig(BaseSettings):
    """
    File based registry configs.
    """

    repository_root: str = "."


class ServerConfig(BaseSettings):
    """
    Server (GRPC server + REST server) configuration
    """

    # Server metadata for predict API
    server_name: str = "furiosa-server"
    server_version: str = __version__
    extensions: List[str] = []

    # Repository/Registry configs
    intial_model_autoload: bool = True
    file_registry_config: FileRegistryConfig = FileRegistryConfig()

    # Internal server configs
    grpc_server_config: GRPCServerConfig = GRPCServerConfig()
    rest_server_config: RESTServerConfig = RESTServerConfig()


class ModelConfig(BaseSettings):
    """
    Model configuration for a Nux model
    """

    # Model property for Nux
    name: str
    model: Union[StrictStr, StrictBytes]  # File name string or file binary bytes
    version: Optional[str] = None
    description: Optional[str] = None

    npu_device: Optional[str] = None
    compiler_config: Optional[Dict] = None

    # Model metadata for repository API
    platform: str = "nux"
    versions: Optional[List[str]] = []
    inputs: Optional[List[MetadataTensor]] = []
    outputs: Optional[List[MetadataTensor]] = []



import asyncio
import signal
from typing import List

from .api.grpc.server import GRPCServer
from .api.rest.server import RESTServer
from .handlers import PredictHandler, RepositoryHandler
from .model import NuxModel
from .registry import FileRegistry, InMemoryRegistry
from .repository import Repository
from .settings import ModelConfig, ServerConfig


class ModelServer:
    def __init__(self, config: ServerConfig, model_configs: List[ModelConfig]):
        self._config = config
        self._model_configs = model_configs

        self._repository = Repository(
            [FileRegistry(config.file_registry_config), InMemoryRegistry(model_configs)]
        )

        self._predict_handler = PredictHandler(config, self._repository)
        self._repository_handler = RepositoryHandler(self._repository)

        self._rest_server = RESTServer(
            config.rest_server_config, self._predict_handler, self._repository_handler
        )

        self._grpc_server = GRPCServer(
            config.grpc_server_config, self._predict_handler, self._repository_handler
        )

    async def start(self):
        # Signal handling
        loop = asyncio.get_event_loop()
        for sign in (signal.SIGTERM, signal.SIGINT):
            # Add signal handler for terminating GRPC/REST server at once
            loop.add_signal_handler(sign, lambda: asyncio.ensure_future(self.stop()))

        # Model loading
        if self._config.intial_model_autoload:
            await self.load()

        # Run servers
        await asyncio.gather(self._rest_server.start(), self._grpc_server.start())

    async def stop(self):
        await asyncio.gather(self._rest_server.stop(), self._grpc_server.stop())

    async def load(self):
        for config in await self._repository.list():
            # TODO(yan): Support other model implementation
            await self._repository.load(NuxModel(config))



from typing import Callable

import grpc

from ...errors import ModelServerError
from ...handlers import PredictHandler, RepositoryHandler
from .converters import (
    ModelInferRequestConverter,
    ModelInferResponseConverter,
    ModelMetadataResponseConverter,
    RepositoryIndexRequestConverter,
    RepositoryIndexResponseConverter,
    ServerMetadataResponseConverter,
)
from .generated import model_repository_pb2 as mr_pb
from .generated import predict_pb2 as pb
from .generated.model_repository_pb2_grpc import ModelRepositoryServiceServicer
from .generated.predict_pb2_grpc import GRPCInferenceServiceServicer


def _handle_error(f: Callable):
    async def _inner(self, request, context):
        try:
            return await f(self, request, context)
        except ModelServerError as err:
            await context.abort(code=grpc.StatusCode.INVALID_ARGUMENT, details=str(err))

    return _inner


class InferenceServicer(GRPCInferenceServiceServicer):
    def __init__(self, predict_handler: PredictHandler):
        super().__init__()
        self._predict_handler = predict_handler

    async def ServerLive(self, request: pb.ServerLiveRequest, context) -> pb.ServerLiveResponse:
        is_live = await self._predict_handler.live()
        return pb.ServerLiveResponse(live=is_live)

    async def ServerReady(self, request: pb.ServerReadyRequest, context) -> pb.ServerReadyResponse:
        is_ready = await self._predict_handler.ready()
        return pb.ServerReadyResponse(ready=is_ready)

    async def ModelReady(self, request: pb.ModelReadyRequest, context) -> pb.ModelReadyResponse:
        is_model_ready = await self._predict_handler.model_ready(
            name=request.name, version=request.version
        )
        return pb.ModelReadyResponse(ready=is_model_ready)

    async def ServerMetadata(
        self, request: pb.ServerMetadataRequest, context
    ) -> pb.ServerMetadataResponse:
        metadata = await self._predict_handler.metadata()
        return ServerMetadataResponseConverter.from_types(metadata)

    @_handle_error
    async def ModelMetadata(
        self, request: pb.ModelMetadataRequest, context
    ) -> pb.ModelMetadataResponse:
        metadata = await self._predict_handler.model_metadata(
            name=request.name, version=request.version
        )
        return ModelMetadataResponseConverter.from_types(metadata)

    @_handle_error
    async def ModelInfer(self, request: pb.ModelInferRequest, context) -> pb.ModelInferResponse:
        payload = ModelInferRequestConverter.to_types(request)
        result = await self._predict_handler.infer(
            payload=payload, name=request.model_name, version=request.model_version
        )
        response = ModelInferResponseConverter.from_types(result)
        return response


class ModelRepositoryServicer(ModelRepositoryServiceServicer):
    def __init__(self, handler: RepositoryHandler):
        self._handler = handler

    async def RepositoryIndex(
        self, request: mr_pb.RepositoryIndexRequest, context
    ) -> mr_pb.RepositoryIndexResponse:
        payload = RepositoryIndexRequestConverter.to_types(request)
        index = await self._handler.index(payload)
        return RepositoryIndexResponseConverter.from_types(index)

    @_handle_error
    async def RepositoryModelLoad(
        self, request: mr_pb.RepositoryModelLoadRequest, context
    ) -> mr_pb.RepositoryModelLoadResponse:
        await self._handler.load(request.model_name)
        return mr_pb.RepositoryModelLoadResponse()

    @_handle_error
    async def RepositoryModelUnload(
        self, request: mr_pb.RepositoryModelUnloadRequest, context
    ) -> mr_pb.RepositoryModelUnloadResponse:
        await self._handler.unload(request.model_name)
        return mr_pb.RepositoryModelUnloadResponse()



"""
Model class for prediction/explanation
"""

from abc import ABC, abstractmethod
from typing import Any, List, Optional, overload

import numpy as np

from furiosa.common.thread import asynchronous
from furiosa.runtime import session
from furiosa.runtime.tensor import TensorArray, TensorDesc

from .settings import ModelConfig
from .types import (
    InferenceRequest,
    InferenceResponse,
    MetadataModelResponse,
    RequestInput,
    ResponseOutput,
)


class Model(ABC):
    """
    Base model class for every runtime
    """

    def __init__(self, config: ModelConfig):
        self.ready = False
        self._config = config

    @property
    def name(self) -> str:
        return self._config.name

    @property
    def version(self) -> Optional[str]:
        return self._config.version

    async def metadata(self) -> MetadataModelResponse:
        # FIXME(yan): Model metadata is not currently provided
        return MetadataModelResponse(
            name=self.name,
            platform=self._config.platform,
            versions=[self._config.version or "default"],
            inputs=self._config.inputs,
            outputs=self._config.outputs,
        )

    async def load(self) -> bool:
        self.ready = True
        return self.ready

    async def unload(self):
        self.ready = False

    @overload
    async def predict(self, payload: InferenceRequest) -> InferenceResponse:
        ...

    @overload
    async def predict(self, payload: List[np.ndarray]) -> List[np.ndarray]:
        ...

    @abstractmethod
    async def predict(self, payload):
        pass

    @abstractmethod
    def encode(self, metatdata: Any, payload: Any) -> ResponseOutput:
        pass

    @abstractmethod
    def decode(self, metatdata: Any, request_input: RequestInput) -> Any:
        pass


class NuxModel(Model):
    """
    Model for Nux runtime
    """

    async def predict(self, payload):
        if isinstance(payload, InferenceRequest):
            # Python list to Numpy array
            inputs = [
                self.decode(tensor, request)
                for tensor, request in zip(self._session.inputs(), payload.inputs)
            ]
        else:
            inputs: List[np.ndarray] = payload

        # Infer from Nux
        tensors = [tensor.numpy() for tensor in await self.run(inputs)]

        if isinstance(payload, InferenceRequest):
            # TensorArray(Numpy array) to Python list
            outputs = [
                # TODO(yan): Support named tensor. See furiosa-ai/npu-tools#2421
                self.encode(str(i), tensor)
                for i, tensor in enumerate(tensors)
            ]

            return InferenceResponse(
                model_name=self.name, model_version=self.version, outputs=outputs
            )
        else:
            return tensors

    async def run(self, inputs: List[np.ndarray]) -> TensorArray:
        return await self.session.run(inputs)

    async def load(self) -> bool:
        if self.ready:
            return True

        # TODO(yan): Wrap functions with async thread now. Replace the functions itself
        self._session = await asynchronous(session.create)(
            self._config.model,
            device=self._config.npu_device,
            compile_config=self._config.compiler_config,
        )
        self._session.run = asynchronous(self._session.run)
        return await super().load()

    async def unload(self):
        if not self.ready:
            return

        self._session.close()
        await super().unload()

    @property
    def session(self) -> session.Session:
        assert self.ready is True, "Could not access session unless model loaded first"
        return self._session

    # TODO(yan): Extract codecs to support other type conversion
    def encode(self, name: str, payload: np.ndarray) -> ResponseOutput:
        return ResponseOutput(
            name=name,
            # TODO(yan): Add datatype dictionary for "BYTES: byte"
            datatype=str(payload.dtype).upper(),
            shape=list(payload.shape),
            data=payload.flatten().tolist(),
        )

    def decode(self, tensor: TensorDesc, request_input: RequestInput) -> np.ndarray:
        return np.array(request_input.data, dtype=tensor.numpy_dtype).reshape(tensor.shape)



from concurrent.futures import ThreadPoolExecutor

from grpc import aio

from ...handlers import PredictHandler, RepositoryHandler
from ...settings import GRPCServerConfig
from .generated.model_repository_pb2_grpc import add_ModelRepositoryServiceServicer_to_server
from .generated.predict_pb2_grpc import add_GRPCInferenceServiceServicer_to_server
from .servicers import InferenceServicer, ModelRepositoryServicer


class GRPCServer:
    def __init__(
        self,
        config: GRPCServerConfig,
        predict_handler: PredictHandler,
        repository_handler: RepositoryHandler,
    ):
        self._config = config
        self._predict_handler = predict_handler
        self._repository_handlers = repository_handler

    def _create_server(self):
        self._inference_servicer = InferenceServicer(self._predict_handler)
        self._model_repository_servicer = ModelRepositoryServicer(self._repository_handlers)
        self._server = aio.server(ThreadPoolExecutor(max_workers=self._config.workers))

        add_GRPCInferenceServiceServicer_to_server(self._inference_servicer, self._server)
        add_ModelRepositoryServiceServicer_to_server(self._model_repository_servicer, self._server)

        self._server.add_insecure_port(f"{self._config.host}:{self._config.port}")

        return self._server

    async def start(self):
        self._create_server()

        await self._server.start()
        await self._server.wait_for_termination()

    async def stop(self):
        await self._server.stop(grace=5)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

# FIXME(yan): This is manually modified
from . import model_repository_pb2 as model__repository__pb2


class ModelRepositoryServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RepositoryIndex = channel.unary_unary(
            '/inference.model_repository.ModelRepositoryService/RepositoryIndex',
            request_serializer=model__repository__pb2.RepositoryIndexRequest.SerializeToString,
            response_deserializer=model__repository__pb2.RepositoryIndexResponse.FromString,
        )
        self.RepositoryModelLoad = channel.unary_unary(
            '/inference.model_repository.ModelRepositoryService/RepositoryModelLoad',
            request_serializer=model__repository__pb2.RepositoryModelLoadRequest.SerializeToString,
            response_deserializer=model__repository__pb2.RepositoryModelLoadResponse.FromString,
        )
        self.RepositoryModelUnload = channel.unary_unary(
            '/inference.model_repository.ModelRepositoryService/RepositoryModelUnload',
            request_serializer=model__repository__pb2.RepositoryModelUnloadRequest.SerializeToString,
            response_deserializer=model__repository__pb2.RepositoryModelUnloadResponse.FromString,
        )


class ModelRepositoryServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RepositoryIndex(self, request, context):
        """Get the index of model repository contents."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RepositoryModelLoad(self, request, context):
        """Load or reload a model from a repository."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RepositoryModelUnload(self, request, context):
        """Unload a model."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ModelRepositoryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'RepositoryIndex': grpc.unary_unary_rpc_method_handler(
            servicer.RepositoryIndex,
            request_deserializer=model__repository__pb2.RepositoryIndexRequest.FromString,
            response_serializer=model__repository__pb2.RepositoryIndexResponse.SerializeToString,
        ),
        'RepositoryModelLoad': grpc.unary_unary_rpc_method_handler(
            servicer.RepositoryModelLoad,
            request_deserializer=model__repository__pb2.RepositoryModelLoadRequest.FromString,
            response_serializer=model__repository__pb2.RepositoryModelLoadResponse.SerializeToString,
        ),
        'RepositoryModelUnload': grpc.unary_unary_rpc_method_handler(
            servicer.RepositoryModelUnload,
            request_deserializer=model__repository__pb2.RepositoryModelUnloadRequest.FromString,
            response_serializer=model__repository__pb2.RepositoryModelUnloadResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'inference.model_repository.ModelRepositoryService', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ModelRepositoryService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RepositoryIndex(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.model_repository.ModelRepositoryService/RepositoryIndex',
            model__repository__pb2.RepositoryIndexRequest.SerializeToString,
            model__repository__pb2.RepositoryIndexResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RepositoryModelLoad(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.model_repository.ModelRepositoryService/RepositoryModelLoad',
            model__repository__pb2.RepositoryModelLoadRequest.SerializeToString,
            model__repository__pb2.RepositoryModelLoadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RepositoryModelUnload(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/inference.model_repository.ModelRepositoryService/RepositoryModelUnload',
            model__repository__pb2.RepositoryModelUnloadRequest.SerializeToString,
            model__repository__pb2.RepositoryModelUnloadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# generated by datamodel-codegen:
#   filename:  model_repository.yaml
#   timestamp: 2021-07-26T09:49:58+00:00

from __future__ import annotations

from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, Field


class RepositoryIndexRequest(BaseModel):
    ready: Optional[bool] = None


class State(Enum):
    UNKNOWN = 'UNKNOWN'
    READY = 'READY'
    UNAVAILABLE = 'UNAVAILABLE'
    LOADING = 'LOADING'
    UNLOADING = 'UNLOADING'


class RepositoryIndexResponseItem(BaseModel):
    name: str
    version: Optional[str] = None
    state: State
    reason: str


class RepositoryIndexResponse(BaseModel):
    __root__: List[RepositoryIndexResponseItem] = Field(..., title='repository_index_response')

    # FIXME(yan): This was manually added. Replace codegen template later
    def __iter__(self):
        return iter(self.__root__)

    def __getitem__(self, idx):
        return self.__root__[idx]

    def __len__(self):
        return len(self.__root__)


class RepositoryIndexErrorResponse(BaseModel):
    error: Optional[str] = None


class RepositoryLoadErrorResponse(BaseModel):
    error: Optional[str] = None


class RepositoryUnloadErrorResponse(BaseModel):
    error: Optional[str] = None



from typing import Any, Mapping, Optional, Union

from ... import types
from .generated import model_repository_pb2 as mr_pb
from .generated import predict_pb2 as pb

_FIELDS = {
    "BOOL": "bool_contents",
    "UINT8": "uint_contents",
    "UINT16": "uint_contents",
    "UINT32": "uint_contents",
    "UINT64": "uint64_contents",
    "INT8": "int_contents",
    "INT16": "int_contents",
    "INT32": "int_contents",
    "INT64": "int64_contents",
    "FP16": "bytes_contents",
    "FP32": "fp32_contents",
    "FP64": "fp64_contents",
    "BYTES": "bytes_contents",
}


def _get_value(pb_object, default: Any = None) -> Any:
    fields = pb_object.ListFields()
    if len(fields) == 0:
        return default

    _, field_value = fields[0]
    return field_value


def _merge_map(pb_map: Mapping, value_dict: Mapping) -> Mapping:
    for key, value in value_dict.items():
        pb_map[key].MergeFrom(value)

    return pb_map


class ServerMetadataResponseConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ServerMetadataResponse) -> types.MetadataServerResponse:
        pass

    @classmethod
    def from_types(cls, type_object: types.MetadataServerResponse) -> pb.ServerMetadataResponse:
        return pb.ServerMetadataResponse(
            name=type_object.name,
            version=type_object.version,
            extensions=type_object.extensions,
        )


class ModelMetadataResponseConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelMetadataResponse) -> types.MetadataModelResponse:
        pass

    @classmethod
    def from_types(cls, type_object: types.MetadataModelResponse) -> pb.ModelMetadataResponse:
        metadata = pb.ModelMetadataResponse(
            name=type_object.name,
            platform=type_object.platform,
            versions=type_object.versions,
        )

        if type_object.inputs is not None:
            metadata.inputs.extend(
                [TensorMetadataConverter.from_types(inp) for inp in type_object.inputs]
            )

        if type_object.outputs is not None:
            metadata.outputs.extend(
                [TensorMetadataConverter.from_types(out) for out in type_object.outputs]
            )

        return metadata


class TensorMetadataConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelMetadataResponse.TensorMetadata) -> types.MetadataTensor:
        pass

    @classmethod
    def from_types(
        cls, type_object: types.MetadataTensor
    ) -> pb.ModelMetadataResponse.TensorMetadata:
        tensor_metadata = pb.ModelMetadataResponse.TensorMetadata(
            name=type_object.name,
            datatype=type_object.datatype,
            shape=type_object.shape,
        )

        if type_object.tags is not None:
            # NOTE: We use the ParametersConverter here because it has a
            # similar signature to Tags
            _merge_map(
                tensor_metadata.tags,
                ParametersConverter.from_types(type_object.tags),  # type: ignore
            )

        return tensor_metadata


class ModelInferRequestConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelInferRequest) -> types.InferenceRequest:
        inference_request = types.InferenceRequest.construct(
            id=pb_object.id,
            parameters=ParametersConverter.to_types(pb_object.parameters),
            inputs=[InferInputTensorConverter.to_types(inp) for inp in pb_object.inputs],
        )

        if pb_object.outputs:
            inference_request.outputs = [
                InferRequestedOutputTensorConverter.to_types(out) for out in pb_object.outputs
            ]

        return inference_request

    @classmethod
    def from_types(
        cls, type_object: types.InferenceRequest, model_name: str, model_version: str
    ) -> pb.ModelInferRequest:
        model_infer_request = pb.ModelInferRequest(
            model_name=model_name,
            model_version=model_version,
            inputs=[InferInputTensorConverter.from_types(inp) for inp in type_object.inputs],
        )

        if type_object.id is not None:
            model_infer_request.id = type_object.id

        if type_object.parameters is not None:
            _merge_map(
                model_infer_request.parameters,
                ParametersConverter.from_types(type_object.parameters),
            )

        if type_object.outputs is not None:
            model_infer_request.outputs.extend(
                [InferRequestedOutputTensorConverter.from_types(out) for out in type_object.outputs]
            )

        return model_infer_request


class InferInputTensorConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelInferRequest.InferInputTensor) -> types.RequestInput:
        return types.RequestInput.construct(
            name=pb_object.name,
            shape=list(pb_object.shape),
            datatype=pb_object.datatype,
            parameters=ParametersConverter.to_types(pb_object.parameters),
            data=InferTensorContentsConverter.to_types(pb_object.contents),
        )

    @classmethod
    def from_types(cls, type_object: types.RequestInput) -> pb.ModelInferRequest.InferInputTensor:
        infer_input_tensor = pb.ModelInferRequest.InferInputTensor(
            name=type_object.name,
            shape=type_object.shape,
            datatype=type_object.datatype,
            contents=InferTensorContentsConverter.from_types(
                type_object.data, datatype=type_object.datatype
            ),
        )

        if type_object.parameters is not None:
            _merge_map(
                infer_input_tensor.parameters,
                ParametersConverter.from_types(type_object.parameters),
            )

        return infer_input_tensor


class InferRequestedOutputTensorConverter:
    @classmethod
    def to_types(
        cls, pb_object: pb.ModelInferRequest.InferRequestedOutputTensor
    ) -> types.RequestOutput:
        return types.RequestOutput.construct(
            name=pb_object.name,
            parameters=ParametersConverter.to_types(pb_object.parameters),
        )

    @classmethod
    def from_types(
        cls, type_object: types.RequestOutput
    ) -> pb.ModelInferRequest.InferRequestedOutputTensor:
        model_infer_request = pb.ModelInferRequest.InferRequestedOutputTensor(
            name=type_object.name,
        )

        if type_object.parameters is not None:
            _merge_map(
                model_infer_request.parameters,
                ParametersConverter.from_types(type_object.parameters),
            )

        return model_infer_request


class ParametersConverter:
    @classmethod
    def to_types(cls, pb_object: Mapping[str, pb.InferParameter]) -> Optional[types.Parameters]:
        if not pb_object:
            return None

        param_dict = {
            key: _get_value(infer_parameter) for key, infer_parameter in pb_object.items()
        }
        return types.Parameters(**param_dict)

    @classmethod
    def from_types(cls, type_object: types.Parameters) -> Mapping[str, pb.InferParameter]:
        pb_object = {}
        as_dict = type_object.dict()

        for key, value in as_dict.items():
            infer_parameter_key = cls._get_inferparameter_key(value)
            if infer_parameter_key is None:
                # TODO: Log warning about ignored field
                continue

            infer_parameter = pb.InferParameter(**{infer_parameter_key: value})
            pb_object[key] = infer_parameter

        return pb_object

    @classmethod
    def _get_inferparameter_key(cls, value: Union[bool, str, int]) -> Optional[str]:
        if isinstance(value, bool):
            return "bool_param"
        elif isinstance(value, str):
            return "string_param"
        elif isinstance(value, int):
            return "int64_param"

        return None


class InferTensorContentsConverter:
    @classmethod
    def to_types(cls, pb_object: pb.InferTensorContents) -> types.TensorData:
        data = _get_value(pb_object, default=[])
        return types.TensorData.construct(__root__=data)

    @classmethod
    def from_types(cls, type_object: types.TensorData, datatype: str) -> pb.InferTensorContents:
        contents = cls._get_contents(type_object, datatype)
        return pb.InferTensorContents(**contents)

    @classmethod
    def _get_contents(cls, type_object: types.TensorData, datatype: str) -> dict:
        field = _FIELDS[datatype]
        return {field: type_object}


class ModelInferResponseConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelInferResponse) -> types.InferenceResponse:
        pass

    @classmethod
    def from_types(cls, type_object: types.InferenceResponse) -> pb.ModelInferResponse:
        model_infer_response = pb.ModelInferResponse(
            model_name=type_object.model_name,
            outputs=[
                InferOutputTensorConverter.from_types(output) for output in type_object.outputs
            ],
        )

        if type_object.model_version is not None:
            model_infer_response.model_version = type_object.model_version

        if type_object.id is not None:
            model_infer_response.id = type_object.id

        if type_object.parameters:
            _merge_map(
                model_infer_response.parameters,
                ParametersConverter.from_types(type_object.parameters),
            )

        return model_infer_response


class InferOutputTensorConverter:
    @classmethod
    def to_types(cls, pb_object: pb.ModelInferResponse.InferOutputTensor) -> types.ResponseOutput:
        pass

    @classmethod
    def from_types(
        cls, type_object: types.ResponseOutput
    ) -> pb.ModelInferResponse.InferOutputTensor:
        infer_output_tensor = pb.ModelInferResponse.InferOutputTensor(
            name=type_object.name,
            shape=type_object.shape,
            datatype=type_object.datatype,
            contents=InferTensorContentsConverter.from_types(
                type_object.data, datatype=type_object.datatype
            ),
        )

        if type_object.parameters:
            _merge_map(
                infer_output_tensor.parameters,
                ParametersConverter.from_types(type_object.parameters),
            )

        return infer_output_tensor


class RepositoryIndexRequestConverter:
    @classmethod
    def to_types(cls, pb_object: mr_pb.RepositoryIndexRequest) -> types.RepositoryIndexRequest:
        return types.RepositoryIndexRequest(
            ready=pb_object.ready,
        )

    @classmethod
    def from_types(cls, type_object: types.RepositoryIndexRequest) -> mr_pb.RepositoryIndexRequest:
        raise NotImplementedError("")


class RepositoryIndexResponseConverter:
    @classmethod
    def to_types(cls, pb_object: mr_pb.RepositoryIndexResponse) -> types.RepositoryIndexResponse:
        raise NotImplementedError("")

    @classmethod
    def from_types(
        cls, type_object: types.RepositoryIndexResponse
    ) -> mr_pb.RepositoryIndexResponse:
        return mr_pb.RepositoryIndexResponse(
            models=[RepositoryIndexResponseItemConverter.from_types(model) for model in type_object]
        )


class RepositoryIndexResponseItemConverter:
    @classmethod
    def to_types(
        cls, pb_object: mr_pb.RepositoryIndexResponse.ModelIndex
    ) -> types.RepositoryIndexResponseItem:
        raise NotImplementedError("")

    @classmethod
    def from_types(
        cls, type_object: types.RepositoryIndexResponseItem
    ) -> mr_pb.RepositoryIndexResponse.ModelIndex:
        model_index = mr_pb.RepositoryIndexResponse.ModelIndex(
            name=type_object.name,
            state=type_object.state.value,
            reason=type_object.reason,
        )

        if type_object.version is not None:
            model_index.version = type_object.version

        return model_index



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: predict.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name='predict.proto',
    package='inference',
    syntax='proto3',
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\rpredict.proto\x12\tinference\"\x13\n\x11ServerLiveRequest\"\"\n\x12ServerLiveResponse\x12\x0c\n\x04live\x18\x01 \x01(\x08\"\x14\n\x12ServerReadyRequest\"$\n\x13ServerReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\"2\n\x11ModelReadyRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"#\n\x12ModelReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\"\x17\n\x15ServerMetadataRequest\"K\n\x16ServerMetadataResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x12\n\nextensions\x18\x03 \x03(\t\"5\n\x14ModelMetadataRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x9f\x03\n\x15ModelMetadataResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\t\x12\x10\n\x08platform\x18\x03 \x01(\t\x12?\n\x06inputs\x18\x04 \x03(\x0b\x32/.inference.ModelMetadataResponse.TensorMetadata\x12@\n\x07outputs\x18\x05 \x03(\x0b\x32/.inference.ModelMetadataResponse.TensorMetadata\x1a\xd0\x01\n\x0eTensorMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\x12G\n\x04tags\x18\x04 \x03(\x0b\x32\x39.inference.ModelMetadataResponse.TensorMetadata.TagsEntry\x1a\x46\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\"\xee\x06\n\x11ModelInferRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\t\x12@\n\nparameters\x18\x04 \x03(\x0b\x32,.inference.ModelInferRequest.ParametersEntry\x12=\n\x06inputs\x18\x05 \x03(\x0b\x32-.inference.ModelInferRequest.InferInputTensor\x12H\n\x07outputs\x18\x06 \x03(\x0b\x32\x37.inference.ModelInferRequest.InferRequestedOutputTensor\x12\x1a\n\x12raw_input_contents\x18\x07 \x03(\x0c\x1a\x94\x02\n\x10InferInputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\x12Q\n\nparameters\x18\x04 \x03(\x0b\x32=.inference.ModelInferRequest.InferInputTensor.ParametersEntry\x12\x30\n\x08\x63ontents\x18\x05 \x01(\x0b\x32\x1e.inference.InferTensorContents\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1a\xd5\x01\n\x1aInferRequestedOutputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12[\n\nparameters\x18\x02 \x03(\x0b\x32G.inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\"\xd5\x04\n\x12ModelInferResponse\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\t\x12\x41\n\nparameters\x18\x04 \x03(\x0b\x32-.inference.ModelInferResponse.ParametersEntry\x12@\n\x07outputs\x18\x05 \x03(\x0b\x32/.inference.ModelInferResponse.InferOutputTensor\x12\x1b\n\x13raw_output_contents\x18\x06 \x03(\x0c\x1a\x97\x02\n\x11InferOutputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\x12S\n\nparameters\x18\x04 \x03(\x0b\x32?.inference.ModelInferResponse.InferOutputTensor.ParametersEntry\x12\x30\n\x08\x63ontents\x18\x05 \x01(\x0b\x32\x1e.inference.InferTensorContents\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\"i\n\x0eInferParameter\x12\x14\n\nbool_param\x18\x01 \x01(\x08H\x00\x12\x15\n\x0bint64_param\x18\x02 \x01(\x03H\x00\x12\x16\n\x0cstring_param\x18\x03 \x01(\tH\x00\x42\x12\n\x10parameter_choice\"\xd0\x01\n\x13InferTensorContents\x12\x15\n\rbool_contents\x18\x01 \x03(\x08\x12\x14\n\x0cint_contents\x18\x02 \x03(\x05\x12\x16\n\x0eint64_contents\x18\x03 \x03(\x03\x12\x15\n\ruint_contents\x18\x04 \x03(\r\x12\x17\n\x0fuint64_contents\x18\x05 \x03(\x04\x12\x15\n\rfp32_contents\x18\x06 \x03(\x02\x12\x15\n\rfp64_contents\x18\x07 \x03(\x01\x12\x16\n\x0e\x62ytes_contents\x18\x08 \x03(\x0c\x32\xfc\x03\n\x14GRPCInferenceService\x12K\n\nServerLive\x12\x1c.inference.ServerLiveRequest\x1a\x1d.inference.ServerLiveResponse\"\x00\x12N\n\x0bServerReady\x12\x1d.inference.ServerReadyRequest\x1a\x1e.inference.ServerReadyResponse\"\x00\x12K\n\nModelReady\x12\x1c.inference.ModelReadyRequest\x1a\x1d.inference.ModelReadyResponse\"\x00\x12W\n\x0eServerMetadata\x12 .inference.ServerMetadataRequest\x1a!.inference.ServerMetadataResponse\"\x00\x12T\n\rModelMetadata\x12\x1f.inference.ModelMetadataRequest\x1a .inference.ModelMetadataResponse\"\x00\x12K\n\nModelInfer\x12\x1c.inference.ModelInferRequest\x1a\x1d.inference.ModelInferResponse\"\x00\x62\x06proto3',
)


_SERVERLIVEREQUEST = _descriptor.Descriptor(
    name='ServerLiveRequest',
    full_name='inference.ServerLiveRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=28,
    serialized_end=47,
)


_SERVERLIVERESPONSE = _descriptor.Descriptor(
    name='ServerLiveResponse',
    full_name='inference.ServerLiveResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='live',
            full_name='inference.ServerLiveResponse.live',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=49,
    serialized_end=83,
)


_SERVERREADYREQUEST = _descriptor.Descriptor(
    name='ServerReadyRequest',
    full_name='inference.ServerReadyRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=85,
    serialized_end=105,
)


_SERVERREADYRESPONSE = _descriptor.Descriptor(
    name='ServerReadyResponse',
    full_name='inference.ServerReadyResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='ready',
            full_name='inference.ServerReadyResponse.ready',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=107,
    serialized_end=143,
)


_MODELREADYREQUEST = _descriptor.Descriptor(
    name='ModelReadyRequest',
    full_name='inference.ModelReadyRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelReadyRequest.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='version',
            full_name='inference.ModelReadyRequest.version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=145,
    serialized_end=195,
)


_MODELREADYRESPONSE = _descriptor.Descriptor(
    name='ModelReadyResponse',
    full_name='inference.ModelReadyResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='ready',
            full_name='inference.ModelReadyResponse.ready',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=197,
    serialized_end=232,
)


_SERVERMETADATAREQUEST = _descriptor.Descriptor(
    name='ServerMetadataRequest',
    full_name='inference.ServerMetadataRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=234,
    serialized_end=257,
)


_SERVERMETADATARESPONSE = _descriptor.Descriptor(
    name='ServerMetadataResponse',
    full_name='inference.ServerMetadataResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ServerMetadataResponse.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='version',
            full_name='inference.ServerMetadataResponse.version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='extensions',
            full_name='inference.ServerMetadataResponse.extensions',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=259,
    serialized_end=334,
)


_MODELMETADATAREQUEST = _descriptor.Descriptor(
    name='ModelMetadataRequest',
    full_name='inference.ModelMetadataRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelMetadataRequest.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='version',
            full_name='inference.ModelMetadataRequest.version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=336,
    serialized_end=389,
)


_MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY = _descriptor.Descriptor(
    name='TagsEntry',
    full_name='inference.ModelMetadataResponse.TensorMetadata.TagsEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelMetadataResponse.TensorMetadata.TagsEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelMetadataResponse.TensorMetadata.TagsEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=737,
    serialized_end=807,
)

_MODELMETADATARESPONSE_TENSORMETADATA = _descriptor.Descriptor(
    name='TensorMetadata',
    full_name='inference.ModelMetadataResponse.TensorMetadata',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelMetadataResponse.TensorMetadata.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='datatype',
            full_name='inference.ModelMetadataResponse.TensorMetadata.datatype',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='shape',
            full_name='inference.ModelMetadataResponse.TensorMetadata.shape',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='tags',
            full_name='inference.ModelMetadataResponse.TensorMetadata.tags',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=599,
    serialized_end=807,
)

_MODELMETADATARESPONSE = _descriptor.Descriptor(
    name='ModelMetadataResponse',
    full_name='inference.ModelMetadataResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelMetadataResponse.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='versions',
            full_name='inference.ModelMetadataResponse.versions',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='platform',
            full_name='inference.ModelMetadataResponse.platform',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='inputs',
            full_name='inference.ModelMetadataResponse.inputs',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='outputs',
            full_name='inference.ModelMetadataResponse.outputs',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELMETADATARESPONSE_TENSORMETADATA,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=392,
    serialized_end=807,
)


_MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY = _descriptor.Descriptor(
    name='ParametersEntry',
    full_name='inference.ModelInferRequest.InferInputTensor.ParametersEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelInferRequest.InferInputTensor.ParametersEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelInferRequest.InferInputTensor.ParametersEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1394,
)

_MODELINFERREQUEST_INFERINPUTTENSOR = _descriptor.Descriptor(
    name='InferInputTensor',
    full_name='inference.ModelInferRequest.InferInputTensor',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelInferRequest.InferInputTensor.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='datatype',
            full_name='inference.ModelInferRequest.InferInputTensor.datatype',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='shape',
            full_name='inference.ModelInferRequest.InferInputTensor.shape',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='inference.ModelInferRequest.InferInputTensor.parameters',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='contents',
            full_name='inference.ModelInferRequest.InferInputTensor.contents',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1118,
    serialized_end=1394,
)

_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY = _descriptor.Descriptor(
    name='ParametersEntry',
    full_name='inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1394,
)

_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR = _descriptor.Descriptor(
    name='InferRequestedOutputTensor',
    full_name='inference.ModelInferRequest.InferRequestedOutputTensor',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelInferRequest.InferRequestedOutputTensor.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='inference.ModelInferRequest.InferRequestedOutputTensor.parameters',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1397,
    serialized_end=1610,
)

_MODELINFERREQUEST_PARAMETERSENTRY = _descriptor.Descriptor(
    name='ParametersEntry',
    full_name='inference.ModelInferRequest.ParametersEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelInferRequest.ParametersEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelInferRequest.ParametersEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1394,
)

_MODELINFERREQUEST = _descriptor.Descriptor(
    name='ModelInferRequest',
    full_name='inference.ModelInferRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='model_name',
            full_name='inference.ModelInferRequest.model_name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='model_version',
            full_name='inference.ModelInferRequest.model_version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='id',
            full_name='inference.ModelInferRequest.id',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='inference.ModelInferRequest.parameters',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='inputs',
            full_name='inference.ModelInferRequest.inputs',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='outputs',
            full_name='inference.ModelInferRequest.outputs',
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='raw_input_contents',
            full_name='inference.ModelInferRequest.raw_input_contents',
            index=6,
            number=7,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELINFERREQUEST_INFERINPUTTENSOR,
        _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR,
        _MODELINFERREQUEST_PARAMETERSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=810,
    serialized_end=1688,
)


_MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY = _descriptor.Descriptor(
    name='ParametersEntry',
    full_name='inference.ModelInferResponse.InferOutputTensor.ParametersEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelInferResponse.InferOutputTensor.ParametersEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelInferResponse.InferOutputTensor.ParametersEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1394,
)

_MODELINFERRESPONSE_INFEROUTPUTTENSOR = _descriptor.Descriptor(
    name='InferOutputTensor',
    full_name='inference.ModelInferResponse.InferOutputTensor',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='inference.ModelInferResponse.InferOutputTensor.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='datatype',
            full_name='inference.ModelInferResponse.InferOutputTensor.datatype',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='shape',
            full_name='inference.ModelInferResponse.InferOutputTensor.shape',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='inference.ModelInferResponse.InferOutputTensor.parameters',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='contents',
            full_name='inference.ModelInferResponse.InferOutputTensor.contents',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1931,
    serialized_end=2210,
)

_MODELINFERRESPONSE_PARAMETERSENTRY = _descriptor.Descriptor(
    name='ParametersEntry',
    full_name='inference.ModelInferResponse.ParametersEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='inference.ModelInferResponse.ParametersEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='inference.ModelInferResponse.ParametersEntry.value',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1318,
    serialized_end=1394,
)

_MODELINFERRESPONSE = _descriptor.Descriptor(
    name='ModelInferResponse',
    full_name='inference.ModelInferResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='model_name',
            full_name='inference.ModelInferResponse.model_name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='model_version',
            full_name='inference.ModelInferResponse.model_version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='id',
            full_name='inference.ModelInferResponse.id',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='inference.ModelInferResponse.parameters',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='outputs',
            full_name='inference.ModelInferResponse.outputs',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='raw_output_contents',
            full_name='inference.ModelInferResponse.raw_output_contents',
            index=5,
            number=6,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _MODELINFERRESPONSE_INFEROUTPUTTENSOR,
        _MODELINFERRESPONSE_PARAMETERSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1691,
    serialized_end=2288,
)


_INFERPARAMETER = _descriptor.Descriptor(
    name='InferParameter',
    full_name='inference.InferParameter',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='bool_param',
            full_name='inference.InferParameter.bool_param',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='int64_param',
            full_name='inference.InferParameter.int64_param',
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='string_param',
            full_name='inference.InferParameter.string_param',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name='parameter_choice',
            full_name='inference.InferParameter.parameter_choice',
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=2290,
    serialized_end=2395,
)


_INFERTENSORCONTENTS = _descriptor.Descriptor(
    name='InferTensorContents',
    full_name='inference.InferTensorContents',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='bool_contents',
            full_name='inference.InferTensorContents.bool_contents',
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='int_contents',
            full_name='inference.InferTensorContents.int_contents',
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='int64_contents',
            full_name='inference.InferTensorContents.int64_contents',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='uint_contents',
            full_name='inference.InferTensorContents.uint_contents',
            index=3,
            number=4,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='uint64_contents',
            full_name='inference.InferTensorContents.uint64_contents',
            index=4,
            number=5,
            type=4,
            cpp_type=4,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='fp32_contents',
            full_name='inference.InferTensorContents.fp32_contents',
            index=5,
            number=6,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='fp64_contents',
            full_name='inference.InferTensorContents.fp64_contents',
            index=6,
            number=7,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='bytes_contents',
            full_name='inference.InferTensorContents.bytes_contents',
            index=7,
            number=8,
            type=12,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2398,
    serialized_end=2606,
)

_MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY.fields_by_name[
    'value'
].message_type = _INFERPARAMETER
_MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY.containing_type = (
    _MODELMETADATARESPONSE_TENSORMETADATA
)
_MODELMETADATARESPONSE_TENSORMETADATA.fields_by_name[
    'tags'
].message_type = _MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY
_MODELMETADATARESPONSE_TENSORMETADATA.containing_type = _MODELMETADATARESPONSE
_MODELMETADATARESPONSE.fields_by_name['inputs'].message_type = _MODELMETADATARESPONSE_TENSORMETADATA
_MODELMETADATARESPONSE.fields_by_name[
    'outputs'
].message_type = _MODELMETADATARESPONSE_TENSORMETADATA
_MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY.fields_by_name[
    'value'
].message_type = _INFERPARAMETER
_MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY.containing_type = (
    _MODELINFERREQUEST_INFERINPUTTENSOR
)
_MODELINFERREQUEST_INFERINPUTTENSOR.fields_by_name[
    'parameters'
].message_type = _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY
_MODELINFERREQUEST_INFERINPUTTENSOR.fields_by_name['contents'].message_type = _INFERTENSORCONTENTS
_MODELINFERREQUEST_INFERINPUTTENSOR.containing_type = _MODELINFERREQUEST
_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY.fields_by_name[
    'value'
].message_type = _INFERPARAMETER
_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY.containing_type = (
    _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR
)
_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR.fields_by_name[
    'parameters'
].message_type = _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY
_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR.containing_type = _MODELINFERREQUEST
_MODELINFERREQUEST_PARAMETERSENTRY.fields_by_name['value'].message_type = _INFERPARAMETER
_MODELINFERREQUEST_PARAMETERSENTRY.containing_type = _MODELINFERREQUEST
_MODELINFERREQUEST.fields_by_name['parameters'].message_type = _MODELINFERREQUEST_PARAMETERSENTRY
_MODELINFERREQUEST.fields_by_name['inputs'].message_type = _MODELINFERREQUEST_INFERINPUTTENSOR
_MODELINFERREQUEST.fields_by_name[
    'outputs'
].message_type = _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR
_MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY.fields_by_name[
    'value'
].message_type = _INFERPARAMETER
_MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY.containing_type = (
    _MODELINFERRESPONSE_INFEROUTPUTTENSOR
)
_MODELINFERRESPONSE_INFEROUTPUTTENSOR.fields_by_name[
    'parameters'
].message_type = _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY
_MODELINFERRESPONSE_INFEROUTPUTTENSOR.fields_by_name['contents'].message_type = _INFERTENSORCONTENTS
_MODELINFERRESPONSE_INFEROUTPUTTENSOR.containing_type = _MODELINFERRESPONSE
_MODELINFERRESPONSE_PARAMETERSENTRY.fields_by_name['value'].message_type = _INFERPARAMETER
_MODELINFERRESPONSE_PARAMETERSENTRY.containing_type = _MODELINFERRESPONSE
_MODELINFERRESPONSE.fields_by_name['parameters'].message_type = _MODELINFERRESPONSE_PARAMETERSENTRY
_MODELINFERRESPONSE.fields_by_name['outputs'].message_type = _MODELINFERRESPONSE_INFEROUTPUTTENSOR
_INFERPARAMETER.oneofs_by_name['parameter_choice'].fields.append(
    _INFERPARAMETER.fields_by_name['bool_param']
)
_INFERPARAMETER.fields_by_name['bool_param'].containing_oneof = _INFERPARAMETER.oneofs_by_name[
    'parameter_choice'
]
_INFERPARAMETER.oneofs_by_name['parameter_choice'].fields.append(
    _INFERPARAMETER.fields_by_name['int64_param']
)
_INFERPARAMETER.fields_by_name['int64_param'].containing_oneof = _INFERPARAMETER.oneofs_by_name[
    'parameter_choice'
]
_INFERPARAMETER.oneofs_by_name['parameter_choice'].fields.append(
    _INFERPARAMETER.fields_by_name['string_param']
)
_INFERPARAMETER.fields_by_name['string_param'].containing_oneof = _INFERPARAMETER.oneofs_by_name[
    'parameter_choice'
]
DESCRIPTOR.message_types_by_name['ServerLiveRequest'] = _SERVERLIVEREQUEST
DESCRIPTOR.message_types_by_name['ServerLiveResponse'] = _SERVERLIVERESPONSE
DESCRIPTOR.message_types_by_name['ServerReadyRequest'] = _SERVERREADYREQUEST
DESCRIPTOR.message_types_by_name['ServerReadyResponse'] = _SERVERREADYRESPONSE
DESCRIPTOR.message_types_by_name['ModelReadyRequest'] = _MODELREADYREQUEST
DESCRIPTOR.message_types_by_name['ModelReadyResponse'] = _MODELREADYRESPONSE
DESCRIPTOR.message_types_by_name['ServerMetadataRequest'] = _SERVERMETADATAREQUEST
DESCRIPTOR.message_types_by_name['ServerMetadataResponse'] = _SERVERMETADATARESPONSE
DESCRIPTOR.message_types_by_name['ModelMetadataRequest'] = _MODELMETADATAREQUEST
DESCRIPTOR.message_types_by_name['ModelMetadataResponse'] = _MODELMETADATARESPONSE
DESCRIPTOR.message_types_by_name['ModelInferRequest'] = _MODELINFERREQUEST
DESCRIPTOR.message_types_by_name['ModelInferResponse'] = _MODELINFERRESPONSE
DESCRIPTOR.message_types_by_name['InferParameter'] = _INFERPARAMETER
DESCRIPTOR.message_types_by_name['InferTensorContents'] = _INFERTENSORCONTENTS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ServerLiveRequest = _reflection.GeneratedProtocolMessageType(
    'ServerLiveRequest',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERLIVEREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerLiveRequest)
    },
)
_sym_db.RegisterMessage(ServerLiveRequest)

ServerLiveResponse = _reflection.GeneratedProtocolMessageType(
    'ServerLiveResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERLIVERESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerLiveResponse)
    },
)
_sym_db.RegisterMessage(ServerLiveResponse)

ServerReadyRequest = _reflection.GeneratedProtocolMessageType(
    'ServerReadyRequest',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERREADYREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerReadyRequest)
    },
)
_sym_db.RegisterMessage(ServerReadyRequest)

ServerReadyResponse = _reflection.GeneratedProtocolMessageType(
    'ServerReadyResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERREADYRESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerReadyResponse)
    },
)
_sym_db.RegisterMessage(ServerReadyResponse)

ModelReadyRequest = _reflection.GeneratedProtocolMessageType(
    'ModelReadyRequest',
    (_message.Message,),
    {
        'DESCRIPTOR': _MODELREADYREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelReadyRequest)
    },
)
_sym_db.RegisterMessage(ModelReadyRequest)

ModelReadyResponse = _reflection.GeneratedProtocolMessageType(
    'ModelReadyResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MODELREADYRESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelReadyResponse)
    },
)
_sym_db.RegisterMessage(ModelReadyResponse)

ServerMetadataRequest = _reflection.GeneratedProtocolMessageType(
    'ServerMetadataRequest',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERMETADATAREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerMetadataRequest)
    },
)
_sym_db.RegisterMessage(ServerMetadataRequest)

ServerMetadataResponse = _reflection.GeneratedProtocolMessageType(
    'ServerMetadataResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _SERVERMETADATARESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ServerMetadataResponse)
    },
)
_sym_db.RegisterMessage(ServerMetadataResponse)

ModelMetadataRequest = _reflection.GeneratedProtocolMessageType(
    'ModelMetadataRequest',
    (_message.Message,),
    {
        'DESCRIPTOR': _MODELMETADATAREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelMetadataRequest)
    },
)
_sym_db.RegisterMessage(ModelMetadataRequest)

ModelMetadataResponse = _reflection.GeneratedProtocolMessageType(
    'ModelMetadataResponse',
    (_message.Message,),
    {
        'TensorMetadata': _reflection.GeneratedProtocolMessageType(
            'TensorMetadata',
            (_message.Message,),
            {
                'TagsEntry': _reflection.GeneratedProtocolMessageType(
                    'TagsEntry',
                    (_message.Message,),
                    {
                        'DESCRIPTOR': _MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY,
                        '__module__': 'predict_pb2'
                        # @@protoc_insertion_point(class_scope:inference.ModelMetadataResponse.TensorMetadata.TagsEntry)
                    },
                ),
                'DESCRIPTOR': _MODELMETADATARESPONSE_TENSORMETADATA,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelMetadataResponse.TensorMetadata)
            },
        ),
        'DESCRIPTOR': _MODELMETADATARESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelMetadataResponse)
    },
)
_sym_db.RegisterMessage(ModelMetadataResponse)
_sym_db.RegisterMessage(ModelMetadataResponse.TensorMetadata)
_sym_db.RegisterMessage(ModelMetadataResponse.TensorMetadata.TagsEntry)

ModelInferRequest = _reflection.GeneratedProtocolMessageType(
    'ModelInferRequest',
    (_message.Message,),
    {
        'InferInputTensor': _reflection.GeneratedProtocolMessageType(
            'InferInputTensor',
            (_message.Message,),
            {
                'ParametersEntry': _reflection.GeneratedProtocolMessageType(
                    'ParametersEntry',
                    (_message.Message,),
                    {
                        'DESCRIPTOR': _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY,
                        '__module__': 'predict_pb2'
                        # @@protoc_insertion_point(class_scope:inference.ModelInferRequest.InferInputTensor.ParametersEntry)
                    },
                ),
                'DESCRIPTOR': _MODELINFERREQUEST_INFERINPUTTENSOR,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelInferRequest.InferInputTensor)
            },
        ),
        'InferRequestedOutputTensor': _reflection.GeneratedProtocolMessageType(
            'InferRequestedOutputTensor',
            (_message.Message,),
            {
                'ParametersEntry': _reflection.GeneratedProtocolMessageType(
                    'ParametersEntry',
                    (_message.Message,),
                    {
                        'DESCRIPTOR': _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY,
                        '__module__': 'predict_pb2'
                        # @@protoc_insertion_point(class_scope:inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry)
                    },
                ),
                'DESCRIPTOR': _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelInferRequest.InferRequestedOutputTensor)
            },
        ),
        'ParametersEntry': _reflection.GeneratedProtocolMessageType(
            'ParametersEntry',
            (_message.Message,),
            {
                'DESCRIPTOR': _MODELINFERREQUEST_PARAMETERSENTRY,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelInferRequest.ParametersEntry)
            },
        ),
        'DESCRIPTOR': _MODELINFERREQUEST,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelInferRequest)
    },
)
_sym_db.RegisterMessage(ModelInferRequest)
_sym_db.RegisterMessage(ModelInferRequest.InferInputTensor)
_sym_db.RegisterMessage(ModelInferRequest.InferInputTensor.ParametersEntry)
_sym_db.RegisterMessage(ModelInferRequest.InferRequestedOutputTensor)
_sym_db.RegisterMessage(ModelInferRequest.InferRequestedOutputTensor.ParametersEntry)
_sym_db.RegisterMessage(ModelInferRequest.ParametersEntry)

ModelInferResponse = _reflection.GeneratedProtocolMessageType(
    'ModelInferResponse',
    (_message.Message,),
    {
        'InferOutputTensor': _reflection.GeneratedProtocolMessageType(
            'InferOutputTensor',
            (_message.Message,),
            {
                'ParametersEntry': _reflection.GeneratedProtocolMessageType(
                    'ParametersEntry',
                    (_message.Message,),
                    {
                        'DESCRIPTOR': _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY,
                        '__module__': 'predict_pb2'
                        # @@protoc_insertion_point(class_scope:inference.ModelInferResponse.InferOutputTensor.ParametersEntry)
                    },
                ),
                'DESCRIPTOR': _MODELINFERRESPONSE_INFEROUTPUTTENSOR,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelInferResponse.InferOutputTensor)
            },
        ),
        'ParametersEntry': _reflection.GeneratedProtocolMessageType(
            'ParametersEntry',
            (_message.Message,),
            {
                'DESCRIPTOR': _MODELINFERRESPONSE_PARAMETERSENTRY,
                '__module__': 'predict_pb2'
                # @@protoc_insertion_point(class_scope:inference.ModelInferResponse.ParametersEntry)
            },
        ),
        'DESCRIPTOR': _MODELINFERRESPONSE,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.ModelInferResponse)
    },
)
_sym_db.RegisterMessage(ModelInferResponse)
_sym_db.RegisterMessage(ModelInferResponse.InferOutputTensor)
_sym_db.RegisterMessage(ModelInferResponse.InferOutputTensor.ParametersEntry)
_sym_db.RegisterMessage(ModelInferResponse.ParametersEntry)

InferParameter = _reflection.GeneratedProtocolMessageType(
    'InferParameter',
    (_message.Message,),
    {
        'DESCRIPTOR': _INFERPARAMETER,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.InferParameter)
    },
)
_sym_db.RegisterMessage(InferParameter)

InferTensorContents = _reflection.GeneratedProtocolMessageType(
    'InferTensorContents',
    (_message.Message,),
    {
        'DESCRIPTOR': _INFERTENSORCONTENTS,
        '__module__': 'predict_pb2'
        # @@protoc_insertion_point(class_scope:inference.InferTensorContents)
    },
)
_sym_db.RegisterMessage(InferTensorContents)


_MODELMETADATARESPONSE_TENSORMETADATA_TAGSENTRY._options = None
_MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY._options = None
_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY._options = None
_MODELINFERREQUEST_PARAMETERSENTRY._options = None
_MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY._options = None
_MODELINFERRESPONSE_PARAMETERSENTRY._options = None

_GRPCINFERENCESERVICE = _descriptor.ServiceDescriptor(
    name='GRPCInferenceService',
    full_name='inference.GRPCInferenceService',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=2609,
    serialized_end=3117,
    methods=[
        _descriptor.MethodDescriptor(
            name='ServerLive',
            full_name='inference.GRPCInferenceService.ServerLive',
            index=0,
            containing_service=None,
            input_type=_SERVERLIVEREQUEST,
            output_type=_SERVERLIVERESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ServerReady',
            full_name='inference.GRPCInferenceService.ServerReady',
            index=1,
            containing_service=None,
            input_type=_SERVERREADYREQUEST,
            output_type=_SERVERREADYRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ModelReady',
            full_name='inference.GRPCInferenceService.ModelReady',
            index=2,
            containing_service=None,
            input_type=_MODELREADYREQUEST,
            output_type=_MODELREADYRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ServerMetadata',
            full_name='inference.GRPCInferenceService.ServerMetadata',
            index=3,
            containing_service=None,
            input_type=_SERVERMETADATAREQUEST,
            output_type=_SERVERMETADATARESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ModelMetadata',
            full_name='inference.GRPCInferenceService.ModelMetadata',
            index=4,
            containing_service=None,
            input_type=_MODELMETADATAREQUEST,
            output_type=_MODELMETADATARESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ModelInfer',
            full_name='inference.GRPCInferenceService.ModelInfer',
            index=5,
            containing_service=None,
            input_type=_MODELINFERREQUEST,
            output_type=_MODELINFERRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_GRPCINFERENCESERVICE)

DESCRIPTOR.services_by_name['GRPCInferenceService'] = _GRPCINFERENCESERVICE

# @@protoc_insertion_point(module_scope)
